@@ -4,7 +4,8 @@
 #                                                                   #
 #-------------------------------------------------------------------#
 
-cusum <- function(data, sizes, center, std.dev, decision.interval = 5, se.shift = 1, data.name, labels, newdata, newsizes, newlabels, plot = TRUE, ...)
+cusum <- function(data, sizes, center, std.dev, head.start = 0, decision.interval = 5,
+                  se.shift = 1, data.name, labels, newdata, newsizes, newlabels, plot = TRUE, ...)
 {
   call <- match.call()
   if (missing(data))
@@ -21,6 +22,12 @@ cusum <- function(data, sizes, center, std.dev, decision.interval = 5, se.shift
          sizes <- rep(sizes, nrow(data))
       else if(length(sizes) != nrow(data))
               stop("sizes length doesn't match with data") }
+
+  if (decision.interval <= 0)
+      stop("decision.interval must be positive")
+
+  if (head.start < 0 || head.start >= decision.interval)
+      stop("head.start must be non-negative and less than decision.interval")
   
   # used for computing statistics and std.dev
   type <- if(any(sizes==1)) "xbar.one" else "xbar"
@@ -100,13 +107,13 @@ cusum <- function(data, sizes, center, std.dev, decision.interval = 5, se.shift
   #
   z.f <- z - se.shift/2
   cusum.pos <- rep(NA, n)
-  cusum.pos[1] <- max(0, z.f[1])
+  cusum.pos[1] <- max(0, head.start + z.f[1])
   for (i in 2:n)
       cusum.pos[i] <- max(0, cusum.pos[i-1]+z.f[i])
   #
   z.f <- z + se.shift/2
   cusum.neg <- rep(NA, n)
-  cusum.neg[1] <- max(0, -z.f[1])
+  cusum.neg[1] <- max(0, head.start - z.f[1])
   for (i in 2:n)
       cusum.neg[i] <- max(0, cusum.neg[i-1]-z.f[i])
   cusum.neg <- -cusum.neg
@@ -116,6 +123,7 @@ cusum <- function(data, sizes, center, std.dev, decision.interval = 5, se.shift
   object$type <- "cusum"
   object$pos <- cusum.pos 
   object$neg <- cusum.neg
+  object$head.start <- head.start
   object$decision.interval <- decision.interval
   object$se.shift <- se.shift
   object$violations <- violations
@@ -165,7 +173,7 @@ summary.cusum.qcc <- function(object, digits =  getOption("digits"), ...)
                 newdata.name, ":\n", sep = ""))
       print(summary(newstats), digits = digits, ...)
       newsizes <- object$newsizes
-      if(!any(diff(newsizes)))
+      if (length(unique(newsizes)) == 1)
          newsizes <- newsizes[1]
       if(length(newsizes) == 1)
          cat("\nGroup sample size: ", format(newsizes))
@@ -181,6 +189,9 @@ summary.cusum.qcc <- function(object, digits =  getOption("digits"), ...)
       cat("\nNumber of groups: ", length(newstats), "\n")
      }
 
+  if (object$head.start > 0)
+      cat("Head start (std.err.):",
+          signif(object$head.start, digits = digits), "\n")
   cat("\nDecision interval (std.err.):", 
       signif(object$decision.interval, digits = digits), "\n")
   cat("Shift detection  (std. err.):", 
@@ -239,16 +250,16 @@ plot.cusum.qcc <- function(x, add.stats = TRUE, chart.all = TRUE,
 
   oldpar <- par(no.readonly = TRUE)
   if(restore.par) on.exit(par(oldpar))
-  mar <- pmax(oldpar$mar, c(5.1,4.1,4.1,2.1))
+  mar <- pmax(oldpar$mar, c(4.1,4.1,3.1,2.1))
   par(bg  = qcc.options("bg.margin"), 
-      cex = qcc.options("cex"),
-      mar = if(add.stats) pmax(mar, c(8.5,0,0,0)) else mar)
+      cex = oldpar$cex * qcc.options("cex"),
+      mar = if(add.stats) pmax(mar, c(7.6,0,0,0)) else mar)
   
   plot(indices, statistics, type="n",
        ylim = if(!missing(ylim)) ylim 
               else range(cusum.pos, cusum.neg, ldb, udb),
        ylab = "", xlab = if(missing(xlab)) "Group" else xlab, 
-       axes = FALSE, main = main.title)
+       axes = FALSE)
   rect(par("usr")[1], par("usr")[3], par("usr")[2], par("usr")[4], 
        col = qcc.options("bg.figure"))
   axis(1, at = indices, las = axes.las,
@@ -256,15 +267,32 @@ plot.cusum.qcc <- function(x, add.stats = TRUE, chart.all = TRUE,
                    as.character(indices) else names(statistics))
   axis(2, las = axes.las)
   box()
+  top.line <- par("mar")[3] - length(capture.output(cat(main.title)))
+  top.line <- top.line - if(chart.all & (!is.null(newstats))) 0.1 else 0.5
+  mtext(main.title, side = 3, line = top.line,
+        font = par("font.main"), 
+        cex  = qcc.options("cex"), 
+        col  = par("col.main"))
+
   abline(h = 0, lwd = 2)
   abline(h = c(ldb, udb), lty = 2)
+  
   lines(indices, cusum.pos[indices], type = "b", pch=20)
   lines(indices, cusum.neg[indices], type = "b", pch=20) 
 
   mtext(if(missing(ylab)) "Cumulative Sum" else ylab, line=3, side=2)
-  mtext("Above target", srt=90, line=2, side=2, at=0+par("usr")[4]/2, cex=0.8)
-  mtext("Below target", srt=90, line=2, side=2, at=0+par("usr")[3]/2, cex=0.8)
-  mtext(label.bounds, side = 4, at = c(ldb, udb), las = 1, line = 0.1, col = gray(0.3))
+  lab <- "Above target"
+  if (add.stats && object$head.start > 0)
+      lab <- paste(lab, " (start = ", object$head.start, ")", sep = "")
+  mtext(lab, srt=90, line=2, side=2, at=0+par("usr")[4]/2,
+        cex = par("cex")*0.8)
+  lab <- "Below target"
+  if (add.stats && object$head.start > 0)
+      lab <- paste(lab, " (start = ", - object$head.start, ")", sep = "")
+  mtext(lab, srt=90, line=2, side=2, at=0+par("usr")[3]/2,
+        cex = par("cex")*0.8)
+  mtext(label.bounds, side = 4, at = c(ldb, udb), las = 1, line = 0.1, 
+        col = gray(0.3), cex = par("cex"))
 
   if(n.beyond.bounds > 0)
     { if(is.null(qcc.options("beyond.limits")))
@@ -283,9 +311,11 @@ plot.cusum.qcc <- function(x, add.stats = TRUE, chart.all = TRUE,
     { len.obj.stats <- length(stats)
       len.new.stats <- length(newstats)
       abline(v = len.obj.stats + 0.5, lty = 3)
-      mtext(paste("Calibration Data in", data.name), cex=0.8,
+      mtext(# paste("Calibration Data in", data.name),
+            "Calibration data", cex = par("cex")*0.8, 
             at = len.obj.stats/2, line = 0, adj = 0.5)
-      mtext(paste("New Data in", object$newdata.name), cex=0.8, 
+      mtext(# paste("New Data in", object$newdata.name), 
+            "New data", cex = par("cex")*0.8, 
             at = len.obj.stats + len.new.stats/2, line = 0, adj = 0.5)
      }
 
@@ -294,42 +324,43 @@ plot.cusum.qcc <- function(x, add.stats = TRUE, chart.all = TRUE,
       plt <- par()$plt; usr <- par()$usr
       px <- diff(usr[1:2])/diff(plt[1:2])
       xfig <- c(usr[1]-px*plt[1], usr[2]+px*(1-plt[2]))
-      at.col <- xfig[1] + diff(xfig[1:2])*c(0.15, 0.60)
+      at.col <- xfig[1] + diff(xfig[1:2])*c(0.15, 0.55)
+      top.line <- 4.5
       # write info at bottom
       mtext(paste("Number of groups = ", length(statistics), sep = ""),
-            side = 1, line = 5, adj = 0, at = at.col[1],
+            side = 1, line = top.line, adj = 0, at = at.col[1],
             font = qcc.options("font.stats"),
-            cex = qcc.options("cex.stats"))
+            cex = par("cex")*qcc.options("cex.stats"))
       if(length(center) == 1)
          { mtext(paste("Center = ", signif(center[1], digits), sep = ""),
-                 side = 1, line = 6, adj = 0, at = at.col[1],
+                 side = 1, line = top.line+1, adj = 0, at = at.col[1],
             font = qcc.options("font.stats"),
-            cex = qcc.options("cex.stats"))
+            cex = par("cex")*qcc.options("cex.stats"))
          }
        else 
          { mtext("Center is variable", 
-                 side = 1, line = 6, adj = 0, at = at.col[1],
+                 side = 1, line = top.line+1, adj = 0, at = at.col[1],
                  font = qcc.options("font.stats"),
-                 cex = qcc.options("cex.stats"))
+                 cex = par("cex")*qcc.options("cex.stats"))
          }
       mtext(paste("StdDev = ", signif(std.dev, digits), sep = ""),
-            side = 1, line = 7, adj = 0, at = at.col[1],
+            side = 1, line = top.line+2, adj = 0, at = at.col[1],
             font = qcc.options("font.stats"),
-            cex = qcc.options("cex.stats"))
+            cex = par("cex")*qcc.options("cex.stats"))
       mtext(paste("Decision interval (std. err.) =", 
                   signif(object$decision.interval, digits = digits)),
-            side = 1, line = 5, adj = 0, at = at.col[2],
+            side = 1, line = top.line, adj = 0, at = at.col[2],
             font = qcc.options("font.stats"),
-            cex = qcc.options("cex.stats"))
+            cex = par("cex")*qcc.options("cex.stats"))
       mtext(paste("Shift detection (std. err.) =", 
                   signif(object$se.shift, digits = digits)), 
-            side = 1, line = 6, adj = 0, at = at.col[2], 
+            side = 1, line = top.line+1, adj = 0, at = at.col[2], 
             font = qcc.options("font.stats"),
-            cex = qcc.options("cex.stats"))
+            cex = par("cex")*qcc.options("cex.stats"))
       mtext(paste("No. of points beyond boundaries =", n.beyond.bounds),
-            side = 1, line = 7, adj = 0, at = at.col[2],
+            side = 1, line = top.line+2, adj = 0, at = at.col[2],
             font = qcc.options("font.stats"),
-            cex = qcc.options("cex.stats"))
+            cex = par("cex")*qcc.options("cex.stats"))
       }
 
   invisible()
