@@ -28,15 +28,10 @@ ewmaSmooth <- function(x, y, lambda = 0.20, start, ...)
   y <- y[ord]
   n <- length(y)
   if (missing(start)) start <- y[1]
-  
-  S1 <- diag(rep(1,n))
-  for (i in 1:(n-1))
-      {for (j in i:n)
-           { S1[j,i] <- (1-lambda)^(j-i) }}
-            
-  S2 <- (1-lambda)^seq(1,n)
-  z <- lambda*(S1%*%y) + S2*start
-  list(x=x, y=z, lambda=lambda, start=start)
+  z <- c(start, y)
+  for (i in 2:(n + 1))
+    z[i] <- lambda * z[i] + (1 - lambda) * z[i - 1]
+  list(x=x, y=z[-1], lambda=lambda, start=start)
 }
 
 
@@ -192,7 +187,7 @@ summary.ewma.qcc <- function(object, digits =  getOption("digits"), ...)
                 newdata.name, ":\n", sep = ""))
       print(summary(newstats), digits = digits, ...)
       newsizes <- object$newsizes
-      if(!any(diff(newsizes)))
+      if (length(unique(newsizes)) == 1)
          newsizes <- newsizes[1]
       if(length(newsizes) == 1)
          cat("\nGroup sample size: ", format(newsizes))
@@ -263,17 +258,17 @@ plot.ewma.qcc <- function(x, add.stats = TRUE, chart.all = TRUE,
 
   oldpar <- par(no.readonly = TRUE)
   if(restore.par) on.exit(par(oldpar))
-  mar <- pmax(oldpar$mar, c(5.1,4.1,4.1,2.1))
+  mar <- pmax(oldpar$mar, c(4.1,4.1,3.1,2.1))
   par(bg  = qcc.options("bg.margin"), 
-      cex = qcc.options("cex"),
-      mar = if(add.stats) pmax(mar, c(8.5,0,0,0)) else mar)
+      cex = oldpar$cex*qcc.options("cex"),
+      mar = if(add.stats) pmax(mar, c(7.6,0,0,0)) else mar)
 
   plot(indices, statistics, type="n",
        ylim = if(!missing(ylim)) ylim 
               else range(statistics, limits),
        ylab = ifelse(missing(ylab), "Group Summary Statistics", ylab),
        xlab = ifelse(missing(xlab), "Group", xlab),
-       axes = FALSE, main = main.title)
+       axes = FALSE)
   rect(par("usr")[1], par("usr")[3], par("usr")[2], par("usr")[4], 
        col = qcc.options("bg.figure"))
   axis(1, at = indices, las = axes.las,
@@ -281,14 +276,22 @@ plot.ewma.qcc <- function(x, add.stats = TRUE, chart.all = TRUE,
                    as.character(indices) else names(statistics))
   axis(2, las = axes.las)
   box()
+  top.line <- par("mar")[3] - length(capture.output(cat(main.title)))
+  top.line <- top.line - if(chart.all & (!is.null(newstats))) 0.1 else 0.5
+  mtext(main.title, side = 3, line = top.line,
+        font = par("font.main"), 
+        cex  = qcc.options("cex"), 
+        col  = par("col.main"))
+
   abline(h = center, lty=1)
   lines(indices, limits[indices,1], lty=2)
   lines(indices, limits[indices,2], lty=2)
+
   points(indices, statistics, pch = 3, cex = 0.8)
   lines(indices, object$y[indices], type = "o", pch=20)
   
   mtext(label.limits, side = 4, at = limits[nrow(limits),], 
-        las = 1, line = 0.1, col = gray(0.3))
+        las = 1, line = 0.1, col = gray(0.3), cex = par("cex"))
 
   if(nviolations > 0)
     { if(is.null(qcc.options("beyond.limits")))
@@ -302,9 +305,11 @@ plot.ewma.qcc <- function(x, add.stats = TRUE, chart.all = TRUE,
     { len.obj.stats <- length(stats)
       len.new.stats <- length(newstats)
       abline(v = len.obj.stats + 0.5, lty = 3)
-      mtext(paste("Calibration Data in", data.name), cex=0.8,
+      mtext(# paste("Calibration Data in", data.name), 
+            "Calibration data", cex = par("cex")*0.8,
             at = len.obj.stats/2, line = 0, adj = 0.5)
-      mtext(paste("New Data in", object$newdata.name), cex=0.8, 
+      mtext(# paste("New Data in", object$newdata.name), 
+            "New data", cex = par("cex")*0.8, 
             at = len.obj.stats + len.new.stats/2, line = 0, adj = 0.5)
      }
 
@@ -314,39 +319,40 @@ plot.ewma.qcc <- function(x, add.stats = TRUE, chart.all = TRUE,
       px <- diff(usr[1:2])/diff(plt[1:2])
       xfig <- c(usr[1]-px*plt[1], usr[2]+px*(1-plt[2]))
       at.col <- xfig[1] + diff(xfig[1:2])*c(0.15, 0.6)
+      top.line <- 4.5
       # write info at bottom
       mtext(paste("Number of groups = ", length(statistics), sep = ""),
-           side = 1, line = 5, adj = 0, at = at.col[1],
+           side = 1, line = top.line, adj = 0, at = at.col[1],
             font = qcc.options("font.stats"),
-            cex = qcc.options("cex.stats"))
+            cex = par("cex")*qcc.options("cex.stats"))
       if(length(center) == 1)
          { mtext(paste("Center = ", signif(center[1], digits = digits), sep = ""),
-                 side = 1, line = 6, adj = 0, at = at.col[1],
+                 side = 1, line = top.line+1, adj = 0, at = at.col[1],
             font = qcc.options("font.stats"),
-            cex = qcc.options("cex.stats"))
+            cex = par("cex")*qcc.options("cex.stats"))
          }
        else 
          { mtext("Center is variable", 
-                 side = 1, line = 6, adj = 0, at = at.col[1],
+                 side = 1, line = top.line+1, adj = 0, at = at.col[1],
                  font = qcc.options("font.stats"),
-                 cex = qcc.options("cex.stats"))
+                 cex = par("cex")*qcc.options("cex.stats"))
          }
       mtext(paste("StdDev = ", signif(std.dev, digits = digits), sep = ""),
-            side = 1, line = 7, adj = 0, at = at.col[1],
+            side = 1, line = top.line+2, adj = 0, at = at.col[1],
             font = qcc.options("font.stats"),
-            cex = qcc.options("cex.stats"))
+            cex = par("cex")*qcc.options("cex.stats"))
       mtext(paste("Smoothing parameter = ", signif(object$lambda, digits = digits)),
-            side = 1, line = 5, adj = 0, at = at.col[2],
+            side = 1, line = top.line, adj = 0, at = at.col[2],
             font = qcc.options("font.stats"),
-            cex = qcc.options("cex.stats"))
+            cex = par("cex")*qcc.options("cex.stats"))
       mtext(paste("Control limits at ", object$nsigmas, "*sigma", sep=""),  
-            side = 1, line = 6, adj = 0, at = at.col[2], 
+            side = 1, line = top.line+1, adj = 0, at = at.col[2], 
             font = qcc.options("font.stats"),
-            cex = qcc.options("cex.stats"))
+            cex = par("cex")*qcc.options("cex.stats"))
       mtext(paste("No. of points beyond limits =", nviolations),
-            side = 1, line = 7, adj = 0, at = at.col[2],
+            side = 1, line = top.line+2, adj = 0, at = at.col[2],
             font = qcc.options("font.stats"),
-            cex = qcc.options("cex.stats"))
+            cex = par("cex")*qcc.options("cex.stats"))
       }
 
   invisible()
